
using System;

namespace Dizzy
{

	public abstract class FileTransfer
	{
		
		// Questo è il riferimento all'elemento della taskview che
		// creeremo per monitorare il progress. 
		protected Gtk.TreeIter iter;
		
		// La connessione vera e propria al server
		protected SFTPConnection sftp;
		
		// L'etichetta da mettere sul trasferimento
		protected string label;
		
		// Un Globally Unique identifier che identifichi il nostro
		// trasferimento
		protected string GUID;
		
		public delegate void Handler(string src, string dest, int transferredBytes,
		                             int totalBytes, string message);
		
		public FileTransfer (string user, string password, string label)
		{
			// Connessione al server
			this.sftp = new SFTPConnection (user, password);
			this.sftp.Connect ();
	
			// Colleghiamo gli eventi a dei pratici handler
			sftp.TransferStarted += new SFTPConnection.SFTPEvent(OnTransferStarted);
			sftp.TransferProgress += new SFTPConnection.SFTPEvent(OnTransferProgress);
			sftp.TransferStopped += new SFTPConnection.SFTPEvent(OnTransferStopped);
			
			this.label = label;
		}
		
		public virtual void Start () {}
		
		public void Stop ()
		{
			// Queste chiamate dovrebbero essere piuttosto standard in tutti
			// i trasferimenti di file che per il momento ci possiamo immaginare.
			this.sftp.Abort ();
			this.sftp.Disconnect ();
		}
		
		
		/*
		 * HANDLER: Queste funzioni aggiornarenno i progress etichettandoli usando
		 * l'etichetta this.label, che deve essere istanziata dal costruttore
		 */
		public void OnTransferStarted (string src, string dest, int transferredBytes,
		                          int totalBytes, string message)
		{
			GUID =	System.Guid.NewGuid().ToString();
			EventManager.TaskTreeViewAddTask (label, 0, GUID);
			this.iter = EventManager.GetIter (GUID);
		}
		
		public void OnTransferProgress(string src, string dest, int transferredBytes,
		                               int totalBytes, string message)
		{
			int perc = transferredBytes / (totalBytes / 100);
			EventManager.TaskTreeViewSetProgress (this.iter, perc);
		}
		
		public void OnTransferStopped (string src, string dest, int transferredBytes,
		                               int totalBytes, string message)
		{
			EventManager.TaskTreeViewRemove (iter);
		}
	}
}
